@@ -39,7 +39,7 @@ except:
 
 import sabnzbd
 from sabnzbd.decorators import synchronized
-from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, GIGI, VERIFIED_FILE, Status
+from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, GIGI, Status, MEBI
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.encoding import unicoder, latin1
@@ -73,8 +73,10 @@ def safe_lower(txt):
 #------------------------------------------------------------------------------
 def globber(path, pattern='*'):
     """ Do a glob.glob(), disabling the [] pattern in 'path' """
-    return glob.glob(os.path.join(path, pattern).replace('[', '[[]'))
-
+    if pattern:
+        return glob.glob(os.path.join(path, pattern).replace('[', '[[]'))
+    else:
+        return glob.glob(path.replace('[', '[[]'))
 
 #------------------------------------------------------------------------------
 def cat_to_opts(cat, pp=None, script=None, priority=None):
@@ -631,7 +633,7 @@ def from_units(val):
     else:
         return 0.0
 
-def to_units(val, spaces=0, dec_limit=2):
+def to_units(val, spaces=0, dec_limit=2, postfix=''):
     """ Convert number to K/M/G/T/P notation
         Add "spaces" if not ending in letter
         dig_limit==1 show single decimal for M and higher
@@ -660,8 +662,8 @@ def to_units(val, spaces=0, dec_limit=2):
     else:
         decimals = 0
 
-    format = '%%s%%.%sf %%s' % decimals
-    return format % (sign, val, unit)
+    format = '%%s%%.%sf %%s%%s' % decimals
+    return format % (sign, val, unit, postfix)
 
 #------------------------------------------------------------------------------
 def same_file(a, b):
@@ -716,17 +718,6 @@ def split_host(srv):
     return (host, port)
 
 
-#------------------------------------------------------------------------------
-def hostname():
-    """ Return host's pretty name """
-    if sabnzbd.WIN32:
-        return os.environ.get('computername', 'unknown')
-    try:
-        return os.uname()[1]
-    except:
-        return 'unknown'
-
-
 #------------------------------------------------------------------------------
 def check_mount(path):
     """ Return False if volume isn't mounted on Linux or OSX
@@ -797,13 +788,23 @@ def create_dirs(dirpath):
 
 
 @synchronized(DIR_LOCK)
-def move_to_path(path, new_path, unique=True):
-    """ Move a file to a new path, optionally give unique filename """
-    if unique:
-        new_path = get_unique_path(new_path, create_dir=False)
+def move_to_path(path, new_path):
+    """ Move a file to a new path, optionally give unique filename
+        Return (ok, new_path)
+    """
+    ok = True
+    overwrite = cfg.overwrite_files()
+    if overwrite and os.path.exists(new_path):
+        try:
+            os.remove(new_path)
+        except:
+            overwrite = False
+    if not overwrite:
+        new_path = get_unique_filename(new_path)
+
     if new_path:
-        logging.debug("Moving. Old path:%s new path:%s unique?:%s",
-                                                  path,new_path, unique)
+        logging.debug("Moving. Old path:%s new path:%s overwrite?:%s",
+                                                  path, new_path, overwrite)
         try:
             # First try cheap rename
             renamer(path, new_path)
@@ -818,8 +819,8 @@ def move_to_path(path, new_path, unique=True):
                 if not (cfg.marker_file() and cfg.marker_file() in path):
                     logging.error(Ta('Failed moving %s to %s'), path, new_path)
                     logging.info("Traceback: ", exc_info = True)
-                new_path = None
-    return new_path
+                ok = False
+    return ok, new_path
 
 
 @synchronized(DIR_LOCK)
@@ -1000,15 +1001,40 @@ def get_filename(path):
     except:
         return ''
 
+
+def memory_usage():
+    try:
+        # Probably only works on Linux because it uses /proc/<pid>/statm
+        t = open('/proc/%d/statm' % os.getpid())
+        v = t.read().split()
+        t.close()
+        virt = int(_PAGE_SIZE * int(v[0]) / MEBI)
+        res = int(_PAGE_SIZE * int(v[1]) / MEBI)
+        return "V=%sM R=%sM" % (virt, res)
+    except:
+        return ''
+
+try:
+    _PAGE_SIZE = os.sysconf("SC_PAGE_SIZE")
+except:
+    _PAGE_SIZE = 0
+_HAVE_STATM = _PAGE_SIZE and memory_usage()
+
+
 def loadavg():
     """ Return 1, 5 and 15 minute load average of host or "" if not supported
     """
-    if sabnzbd.WIN32 or sabnzbd.DARWIN:
-        return ""
-    try:
-        return "%.2f | %.2f | %.2f" % os.getloadavg()
-    except:
-        return ""
+    p = ''
+    if not sabnzbd.WIN32 and not sabnzbd.DARWIN:
+        opt = cfg.show_sysload()
+        if opt:
+            try:
+                p = '%.2f | %.2f | %.2f' % os.getloadavg()
+            except:
+                pass
+            if opt > 1 and _HAVE_STATM:
+                p = '%s | %s' % (p, memory_usage())
+    return p
 
 
 def format_time_string(seconds, days=0):
@@ -1057,7 +1083,11 @@ def int_conv(value):
 # Diskfree
 if sabnzbd.WIN32:
     # windows diskfree
-    import win32api
+    try:
+        # Careful here, because win32api test hasn't been done yet!
+        import win32api
+    except:
+        pass
     def diskfree(_dir):
         """ Return amount of free diskspace in GBytes
         """
