@@ -27,7 +27,7 @@ import datetime
 import sabnzbd
 from sabnzbd.trylist import TryList
 from sabnzbd.nzbstuff import NzbObject
-from sabnzbd.misc import exit_sab, cat_to_opts, flag_file, \
+from sabnzbd.misc import exit_sab, cat_to_opts, \
                          get_admin_path, remove_all, globber
 from sabnzbd.panic import panic_queue
 import sabnzbd.database as database
@@ -147,7 +147,13 @@ class NzbQueue(TryList):
 
 
     def repair_job(self, folder, new_nzb=None):
-        """ Reconstruct admin for a single job folder, optionally with new NZB """
+        """ Reconstruct admin for a single job folder, optionally with new NZB
+        """
+        def all_verified(path):
+            """ Return True when all sets have been successfully verified """
+            verified = sabnzbd.load_data(VERIFIED_FILE, path, remove=False) or {'x':False}
+            return not bool([True for x in verified if not verified[x]])
+
         name = os.path.basename(folder)
         path = os.path.join(folder, JOB_ADMIN)
         if hasattr(new_nzb, 'filename'):
@@ -155,7 +161,7 @@ class NzbQueue(TryList):
         else:
             filename = ''
         if not filename:
-            if not flag_file(folder, VERIFIED_FILE):
+            if not all_verified(path):
                 filename = globber(path, '*.gz')
             if len(filename) > 0:
                 logging.debug('Repair job %s by reparsing stored NZB', latin1(name))
@@ -170,6 +176,35 @@ class NzbQueue(TryList):
             sabnzbd.add_nzbfile(new_nzb, pp=None, script=None, cat=None, priority=None, nzbname=name, reuse=True)
 
 
+    def send_back(self, nzo):
+        """ Send back job to queue after successful pre-check """
+        try:
+            nzb_path = globber(nzo.workpath, '*.gz')[0]
+        except:
+            logging.debug('Failed to find NZB file after pre-check (%s)', nzo.nzo_id)
+            return
+        from sabnzbd.dirscanner import ProcessSingleFile
+        res, nzo_ids = ProcessSingleFile(nzo.work_name + '.nzb', nzb_path, reuse=True)
+        if res == 0 and nzo_ids:
+            self.replace_in_q(nzo, nzo_ids[0])
+
+
+    @synchronized(NZBQUEUE_LOCK)
+    def replace_in_q(self, nzo, nzo_id):
+        """ Replace nzo by new in at the same spot in the queue, destroy nzo """
+        try:
+            new_nzo = self.get_nzo(nzo_id)
+            pos = self.__nzo_list.index(new_nzo)
+            targetpos = self.__nzo_list.index(nzo)
+            self.__nzo_list[targetpos] = new_nzo
+            self.__nzo_list.pop(pos)
+            del self.__nzo_table[nzo.nzo_id]
+            del nzo
+        except:
+            logging.error('Failed to restart NZB after pre-check (%s)', nzo.nzo_id)
+            logging.info("Traceback: ", exc_info = True)
+            return
+
     @synchronized(NZBQUEUE_LOCK)
     def save(self, save_nzo=None):
         """ Save queue, all nzo's or just the specified one """
@@ -297,8 +332,7 @@ class NzbQueue(TryList):
 
         # If no files are to be downloaded anymore, send to postproc
         if not nzo.files and not nzo.futuretype:
-            sabnzbd.remove_data(nzo.nzo_id, nzo.workpath)
-            sabnzbd.proxy_postproc(nzo)
+            self.end_job(nzo)
             return ''
 
         # Reset try_lists
@@ -399,10 +433,10 @@ class NzbQueue(TryList):
             if nzf:
                 post_done = nzo.remove_nzf(nzf)
                 if post_done:
-                    keep_basic = nzo.finished_files
-                    if keep_basic:
-                        sabnzbd.proxy_postproc(nzo)
-                    self.remove(nzo_id, add_to_history = False, keep_basic=keep_basic)
+                    if nzo.finished_files:
+                        self.end_job(nzo)
+                    else:
+                        self.remove(nzo_id, add_to_history = False, keep_basic=False)
 
 
     @synchronized(NZBQUEUE_LOCK)
@@ -714,11 +748,25 @@ class NzbQueue(TryList):
         """
         if self.actives(grabs=False) < 2 and cfg.autodisconnect():
             # This was the last job, close server connections
-            sabnzbd.downloader.Downloader.do.disconnect()
+            if sabnzbd.downloader.Downloader.do:
+                sabnzbd.downloader.Downloader.do.disconnect()
 
         # Notify assembler to call postprocessor
         if not nzo.deleted:
             nzo.deleted = True
+            if nzo.precheck:
+                nzo.save_attribs()
+                # Check result
+                enough, ratio = nzo.check_quality()
+                if enough:
+                    # Enough data present, do real download
+                    workdir = nzo.downpath
+                    self.cleanup_nzo(nzo, keep_basic=True)
+                    self.send_back(nzo)
+                    return
+                else:
+                    # Not enough data, let postprocessor show it as failed
+                    pass
             Assembler.do.process((nzo, None))
 
 
@@ -740,18 +788,24 @@ class NzbQueue(TryList):
     def queue_info(self, for_cli=False, max_jobs=0):
         bytes_left = 0
         bytes = 0
+        q_size = 0
         pnfo_list = []
         n = 0
         for nzo in self.__nzo_list:
-            pnfo = nzo.gather_info(for_cli = for_cli)
-            if nzo.status != 'Paused':
-                bytes += pnfo[PNFO_BYTES_FIELD]
-                bytes_left += pnfo[PNFO_BYTES_LEFT_FIELD]
-            pnfo_list.append(pnfo)
+            if not max_jobs or n < max_jobs:
+                pnfo = nzo.gather_info(for_cli = for_cli)
+                pnfo_list.append(pnfo)
+                if nzo.status != 'Paused':
+                    bytes += pnfo[PNFO_BYTES_FIELD]
+                    bytes_left += pnfo[PNFO_BYTES_LEFT_FIELD]
+                    q_size += 1
+            elif nzo.status != 'Paused':
+                b, b_left = nzo.total_and_remaining()
+                bytes += b
+                bytes_left += b_left
+                q_size += 1
             n += 1
-            if max_jobs and n >= max_jobs:
-                break
-        return (bytes, bytes_left, pnfo_list)
+        return (bytes, bytes_left, pnfo_list, q_size)
 
 
     @synchronized(NZBQUEUE_LOCK)
@@ -780,6 +834,17 @@ class NzbQueue(TryList):
 
         ArticleCache.do.purge_articles(nzo.saved_articles)
 
+    @synchronized(NZBQUEUE_LOCK)
+    def stop_idle_jobs(self):
+        """ Detect jobs that have zero files left and send them to post processing
+        """
+        empty = []
+        for nzo in self.__nzo_list:
+            if not nzo.futuretype and not nzo.files and nzo.status not in (Status.PAUSED, Status.GRABBING):
+                empty.append(nzo)
+        for nzo in empty:
+            self.end_job(nzo)
+
     def get_urls(self):
         """ Return list of future-types needing URL """
         lst = []
